# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from rich         import pretty, traceback
from rich.console import Console
pretty.install()
traceback.install(show_locals=False)
konsol = Console(log_path=False)

from BTKSorgu  import BTKSorgu
from sys       import argv

def basla():
    print()

    if len(argv) != 2:
        konsol.print("[bold yellow2][!] Lütfen Domain Girin..")
        konsol.print("\n[turquoise2]Örn.: [pale_green1]BTKSorgu redtube.com\n")
        return

    konsol.print(f"[yellow]{BTKSorgu(argv[1])}\n")

if __name__ == "__main__":
    basla()


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

import tkinter as tk
from tkinter  import ttk, messagebox
from base64   import encodebytes
from sv_ttk   import set_theme
from BTKSorgu import BTKSorgu

from BTKSorgu.ekstra import dosya_ver


class KekikGUI(tk.Tk):
    def __init__(self):
        super().__init__()
        set_theme("dark")
        self.title("BTK Sorgu")

        logo_b64 = encodebytes(open(file=dosya_ver("logo.png", 1), mode="rb").read())
        favicon  = tk.PhotoImage(data=logo_b64)
        self.iconphoto(False, favicon)

        self.p_genislik  = max(self.winfo_width(), 400)
        self.p_yukseklik = max(self.winfo_height(), 250)
        self.minsize(width=self.p_genislik, height=self.p_yukseklik)

        self.bind("<Escape>",    lambda _: self.pencereyi_kapat())
        self.bind("<F11>",       lambda _: self.tam_ekran())
        self.bind("<Control-a>", lambda event: self.ctrl_a(event))
        self.bind("<Control-A>", lambda event: self.ctrl_a(event))
        self.protocol("WM_DELETE_WINDOW", self.pencereyi_kapat)

        # !
        SorguAlani(self)
        # !

        # * Sağ Alt Köşeye Pencere Yeniden Boyutlandırma
        __temp = ttk.Frame(self)
        __temp.pack(fill="both", expand=True)
        self.sizegrip = ttk.Sizegrip(__temp)
        self.sizegrip.place(relx=1, rely=1, anchor="se", x=-5, y=-5)

    def tam_ekran(self):
        self.attributes("-fullscreen", not self.attributes("-fullscreen"))

        if self.attributes("-fullscreen"):
            self.sizegrip.place(relx=1, rely=1, anchor="se", x=-5, y=-5)       
        else:
            self.sizegrip.place_forget()

    def ctrl_a(self, event):
        event.widget.select_range(0, "end")
        event.widget.icursor("end")

    def pencereyi_kapat(self):
        if messagebox.askokcancel("👋 Ciao..", "Programı Kapatayım Mı?"):
            self.destroy()


class SorguAlani(ttk.Frame):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent = parent
        self.pack(fill="both", expand=True)

        self.rowconfigure(index=(0, 1, 2), weight=1)
        self.columnconfigure(index=0,      weight=1)

        self.btk_ui()

    def btk_ui(self):
        self.arama_metni = ttk.Entry(self)
        self.arama_metni.insert(0, "Sorgulanacak Domain Giriniz..")
        self.arama_metni.bind("<Return>", lambda _: self.ara_buton_tiklaninca())
        self.arama_metni.grid(row=0, column=0, padx=30, pady=10, sticky="ew")
        self.arama_metni.focus()

        ara_buton = ttk.Button(self, text="Ara", command=lambda: self.ara_buton_tiklaninca())
        ara_buton.grid(row=1, column=0, padx=30, sticky="ew")

        ayrac = ttk.Separator(self, orient="horizontal")
        ayrac.grid(row=2, column=0, pady=(10, 0), padx=30, sticky="ew")

        self.cikti_alani = CiktiAlani(self.parent)

    def ara_buton_tiklaninca(self):
        sorgu = self.arama_metni.get()
        if not sorgu:
            return False

        self.arama_metni.state(["disabled"])
        self.cikti_alani.metin.configure(text="Lütfen Bekleyiniz...", foreground="#EF7F1A")
        self.update()

        btk_sorgu = str(BTKSorgu(sorgu))

        if "engellenmiştir" not in btk_sorgu:
            self.arama_metni.state(["!disabled", "!invalid"])
            self.cikti_alani.metin.configure(text=btk_sorgu, foreground="#17a2b8")            
        else:
            self.arama_metni.state(["!disabled", "invalid"])
            self.cikti_alani.metin.configure(text=btk_sorgu, foreground="#dc3545")

        self.update()
        self.arama_metni.focus()


class CiktiAlani(ttk.Frame):
    def __init__(self, parent):
        super().__init__(parent)
        self.parent = parent
        self.pack(fill="both", expand=True)

        self.rowconfigure(index=0,    weight=1)
        self.columnconfigure(index=0, weight=1)

        self.metin = ttk.Label(self, wraplength=self.parent.p_genislik - 65)
        self.metin.configure(justify="center")
        self.metin.grid(row=0, column=0, pady=20, sticky="ns")


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from requests import Session
from parsel   import Selector

from shutil      import copyfileobj
from pytesseract import image_to_string 
from PIL         import Image
from re          import search

from os import remove

class BTKSorgu(object):
    """
    BTKSorgu : Hedef websitesinin BTK Tarafından Erişim Engeli Sorgusu

    Dönüş
    ----------
        (str):
            {domain}, {karar} Karar Bulunamadı.
    """
    def __init__(self, sorgu_url:str, arka_plan:bool=False):
        """Karar Döndürür"""
        self.arka_plan      = arka_plan
        self.ana_sayfa      = "https://internet2.btk.gov.tr"
        self.sorgu_sayfasi  = "https://internet2.btk.gov.tr/sitesorgu/"
        self.sorgu_url      = search(r"(?:https?://)?(?:www\.)?([^/]+)", sorgu_url).group(1)
        self._gecici_gorsel = "captcha.png"

        self.oturum = Session()

    @property
    def captcha_ver(self):
        """Captcha görselini indirip OCR ile okur"""
        ilk_bakis    = self.oturum.get(self.sorgu_sayfasi, allow_redirects=True)
        captcha_yolu = Selector(ilk_bakis.text).xpath("//div[@class='arama_captcha']/img/@src").get()

        captcha_data = self.oturum.get(f"{self.ana_sayfa}{captcha_yolu}", stream=True)
        with open(self._gecici_gorsel, "wb") as captcha_gorsel:
            copyfileobj(captcha_data.raw, captcha_gorsel)

        return image_to_string(Image.open(self._gecici_gorsel)).strip().replace(" ", "")

    @property
    def karar(self):
        """Captcha ile birlikte sorgu sitesini POST eder"""
        captcha = self.captcha_ver

        karar_sayfasi = self.oturum.post(
            url     = self.sorgu_sayfasi,
            headers = {
                "Content-Type" : "application/x-www-form-urlencoded",
                "Host"         : "internet2.btk.gov.tr",
                "Origin"       : self.ana_sayfa,
                "Referer"      : self.sorgu_sayfasi,
            },
            data    = {
                "deger"         : self.sorgu_url,
                "ipw"           : "",
                "kat"           : "",
                "tr"            : "",
                "eg"            : "",
                "ayrintili"     : 0,
                "submit"        : "Sorgula",
                "security_code" : captcha
            }
        )

        # print(karar_sayfasi.text)
        secici     = Selector(karar_sayfasi.text)
        hatali_kod = secici.xpath("//div[@class='icerik']/ul/li/text()").get()
        erisim_var = secici.xpath("//div[@class='yazi2']/text()").get()
        erisim_yok = secici.xpath("//span[@class='yazi2_2']/text()").get()
        karar      = hatali_kod or erisim_var or erisim_yok or ""

        if self.arka_plan:
            print(f"""

            # Sorgu   : {self.sorgu_url}
            # Captcha : {captcha}
            # Karar   : {karar}

""")
        return karar

    def __repr__(self) -> str:
        """Kararı Döndürür"""
        hatalar = ["Lütfen güvenlik kodunu giriniz.", "Güvenlik kodunu yanlış girdiniz. Lütfen Güvenlik Kodunu resimde gördüğünüz şekilde giriniz."]
        while True:
            karar = self.karar
            if karar not in hatalar:
                remove(self._gecici_gorsel)
                return karar